/*
   Frames Per Second Counter Demo.
   Created by the Programming Ace.
   www.UltimateGameProgramming.com

   The purpose of this demo is to keep track of the frames per second
   with the 3D teapot demo in Direct3D.  For this we will use a timer
   function called GetTickCount() which will return the timer in seconds
   since the application started.
*/


// Demo: Frames Per Second Counter.


#define WIN32_LEAN_AND_MEAN
#define VC_LEANMEAN

#include<stdio.h>
#include<d3dx9.h>


// Function Prototypes...
bool InitializeDirect3D(HWND hwnd, bool fullscreen);
void RenderScene();
void SetRotation();
void SetView();
void SetPerpective();
void ShutdownDirect3D();
void GetFPS();


// Here are our Direct3D objects.
LPDIRECT3D9 Direct3D_Object = NULL;
LPDIRECT3DDEVICE9 D3D_Device = NULL;

LPD3DXMESH Teapot = NULL;                    // Mesh object.
D3DMATERIAL9 TeapotMaterial;                 // Objects material.

// This will hold the world matrix, translation matrix, rotation matrix.
D3DXMATRIXA16 World;
D3DXMATRIX trans;
D3DXMATRIX rot;

// This is the rotation angle.
float angle = 0.0f;

// Scene light source.
D3DLIGHT9 light;

// Our new global window handle.  This will be used in our calculate
// FPS function called GetFPS().
HWND g_hwnd;


LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch(message)
       {
		   case WM_KEYUP:
            // If the user presses the escape key then exit the application.
            if(wParam == VK_ESCAPE)
               PostQuitMessage(0);
            break;

         case WM_DESTROY:
         case WM_CLOSE:
            PostQuitMessage(0);
            break;

            default:
               break;
       }

    return DefWindowProc( hwnd, message, wParam, lParam );
}


int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
   MSG msg;                                           // Message object.
   HWND hwnd;                                         // Handle to the window.
   WNDCLASSEX windowClass;                            // Window class object.
   bool done = false;                                 // false if we should exit.

   // This is the Window class.
   windowClass.cbSize = sizeof(WNDCLASSEX);           // size of the WNDCLASSEX structure.
   windowClass.style = CS_HREDRAW | CS_VREDRAW;       // style of the window.
   windowClass.lpfnWndProc = WndProc;                 // Address to the windows procedure.
   windowClass.cbClsExtra = 0;                        // Extra class information.
   windowClass.cbWndExtra = 0;                        // Extra window information.
   windowClass.hInstance = hInstance;                 // Handle of application Instance.
   windowClass.hIcon = LoadIcon(NULL, IDI_APPLICATION);// Window Icon.
   windowClass.hCursor = LoadCursor(NULL, IDC_ARROW); // Load mouse cursor.
   windowClass.hbrBackground = NULL;                  // Background color.
   windowClass.lpszMenuName = NULL;                   // Menu.
   windowClass.lpszClassName = "UGPClass";            // Name of the window class.
   windowClass.hIconSm = LoadIcon(NULL, IDI_APPLICATION);// Minimized window icon.

   // You must register you class with Windows.
   if(!RegisterClassEx(&windowClass)) return 0;

   // Create the window.
   hwnd = CreateWindowEx(NULL,                                       // The extended style.
                         "UGPClass",                                 // Window class.
                         "Shapes Demo - by The Programming Ace",     // Window name.
                          WS_OVERLAPPEDWINDOW | WS_VISIBLE |         // Window style.
                          WS_SYSMENU |WS_CLIPCHILDREN |              // Window style.
                          WS_CLIPSIBLINGS,                           // Window style.
                          100, 100,                                  // X, Y coords.
                          640, 480,                                  // Window size.
                          NULL,                                      // Handle to parent window.
                          NULL,                                      // Menu.
                          hInstance,                                 // Handle to app instance.
                          NULL);                                     // Pointer to window.


   // If there was an error with creating the window, then close the program.
   if(!hwnd)
      return 0;

   g_hwnd = hwnd; // Assign the hwnd handle to the global handle so we can use
                  // it with the FPS function.

   ShowWindow(hwnd, SW_SHOW);    // Show the window.
   UpdateWindow(hwnd);           // Update its display.

   done = false;  // false = run program, true means stop.

   // Initialize Direct3D.  If all went well then continue.
   if(!InitializeDirect3D(hwnd, false)) done = true;

   // Application loop.
   while(!done)
      {
         if(PeekMessage(&msg, NULL, NULL, NULL, PM_REMOVE))
            {
               // If a quit message is received then stop rendering and quit the app.
               if(msg.message == WM_QUIT)
                  {
                     done = true;
                  }

               TranslateMessage(&msg);  // Translate any messages.
               DispatchMessage(&msg);   // Dispatch any messages.
            }
         else
            {
               RenderScene();          // Render a frame.
            }
      }


    // Here we shutdown Direct 3D.
	ShutdownDirect3D();

   // Here we unregister the window class with the OS.
   UnregisterClass("UGPClass", windowClass.hInstance);

   return (int)msg.wParam;
}


bool InitializeDirect3D(HWND hwnd, bool fullscreen)
{
   // This object will allow us to set the display mode of the screen.
   D3DDISPLAYMODE DisplayMode;

   // This will allow us to set the parameters of the screen.
   D3DPRESENT_PARAMETERS Present_Parameters;

   // This is used to get the capabilities of the hardware.
   D3DCAPS9 D3DCaps;

   // It is always a good idea to clear out memory in object although not necessary.
   ZeroMemory(&Present_Parameters, sizeof(Present_Parameters));

   // Create the Direct3D object to get everything started.
   Direct3D_Object = Direct3DCreate9(D3D_SDK_VERSION);

   // Error checking.  Make sure that it was successful.
   if(Direct3D_Object == NULL)
      {
         MessageBox(NULL, "Error, couldn't initialize DirectX!?!",
                    "Error!", MB_OK);
         return false;
      }

   // This function will get the display mode of the device and place it in DisplayMode.
   if(FAILED(Direct3D_Object->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &DisplayMode)))
      {
         MessageBox(NULL, "Error setting the display mode.", "Error!", MB_OK);
         return false;
      }

   // Get the capabilities of the hardware.
	if(FAILED(Direct3D_Object->GetDeviceCaps(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, &D3DCaps)))
		{
		   return false;
	   }

   // Test which is supported, hardware or software vertex processing.
	DWORD VertexProcessing = 0;

	if(D3DCaps.VertexProcessingCaps != 0)
		VertexProcessing |= D3DCREATE_HARDWARE_VERTEXPROCESSING;
	else
		VertexProcessing |= D3DCREATE_SOFTWARE_VERTEXPROCESSING;

   // Here we are setting the applications parameters...
   if(fullscreen)
      {
         Present_Parameters.Windowed = FALSE;               // Window mode (fullscreen).
         Present_Parameters.BackBufferWidth = DisplayMode.Width;
         Present_Parameters.BackBufferHeight = DisplayMode.Height;
      }
   else
      Present_Parameters.Windowed = TRUE;                   // Window mode (not fullscreen).
   Present_Parameters.SwapEffect = D3DSWAPEFFECT_DISCARD;   // Dealing with animation (see doc).
   Present_Parameters.BackBufferFormat = DisplayMode.Format;// Render to the area of the screen.
   Present_Parameters.BackBufferCount = 1;                  // Number of back buffers.
   Present_Parameters.EnableAutoDepthStencil = TRUE;        // Check documentation.
   Present_Parameters.AutoDepthStencilFormat = D3DFMT_D16;  // Check documentation.
   Present_Parameters.BackBufferWidth = DisplayMode.Width;
   Present_Parameters.BackBufferHeight = DisplayMode.Height;


   // Now we must create the rendering device.
   if(FAILED(Direct3D_Object->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hwnd,
                                           D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                                           &Present_Parameters, &D3D_Device)))
      {
         MessageBox(NULL, "CreateDevice() failed!  Make sure you have DirectX 9.",
                    "Error!", MB_OK);
         return false;
      }

   // One last check to be sure.
   if(D3D_Device == NULL)
      {
         MessageBox(NULL, "D3D_Device is equal to NULL!?!", "Error!", MB_OK);
         return false;
      }

   // Setup the light source.
   light.Type = D3DLIGHT_DIRECTIONAL;
   light.Direction = D3DXVECTOR3(1.0f, 0.5f, 1.0f);

   light.Diffuse.r = light.Diffuse.g = light.Diffuse.b = light.Diffuse.a = 1.0f;
   light.Specular.r = light.Specular.g = light.Specular.b = light.Specular.a = 1.0f;

   D3D_Device->SetLight(0, &light);
   D3D_Device->LightEnable(0, TRUE);

   // Setup the material properties for the teapot.
   ZeroMemory(&TeapotMaterial, sizeof(D3DMATERIAL9));
   TeapotMaterial.Diffuse.r = TeapotMaterial.Ambient.r = 0.6f;
   TeapotMaterial.Diffuse.g = TeapotMaterial.Ambient.g = 0.6f;
   TeapotMaterial.Diffuse.b = TeapotMaterial.Ambient.b = 0.7f;
   TeapotMaterial.Specular.r = 0.4f;
   TeapotMaterial.Specular.g = 0.4f;
   TeapotMaterial.Specular.b = 0.4f;
   TeapotMaterial.Power = 8.0f;

   // First is for ambient lighting.
   D3D_Device->SetRenderState(D3DRS_AMBIENT, D3DCOLOR_COLORVALUE(0.3f, 0.3f, 0.3f, 1.0f));
	D3D_Device->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);// Culling off.
   D3D_Device->SetRenderState(D3DRS_ZENABLE, TRUE);         // Enable depth testing.

   // To create the teapot we simple call D3DXCreateTeapot and the rest is done for us.
   if(FAILED(D3DXCreateTeapot(D3D_Device, &Teapot, NULL)))
      return false;

   // Set the window perpective.
   SetPerpective();

   return true;
}


void SetPerpective()
{
   // This matrix will do nothing more but tell Direct3D how far back it can see
   // and how close it can see.
   D3DXMATRIXA16 Projection;

   // Set the perspective information.
   D3DXMatrixPerspectiveFovLH(&Projection, 45.0f, 640/480, 0.1f, 500.0f);

   // Set the projection.
   D3D_Device->SetTransform(D3DTS_PROJECTION, &Projection);
}


void SetTranslation(float x, float y, float z)
{
   // Set the rotation by mouse.
   D3DXMatrixTranslation(&trans, x, y, z);
}


void SetRotation()
{
   // Set the rotation value to the matrix.
   D3DXMatrixRotationY(&rot, angle);

   // Add 0.01 to angle to rotate the triangle even more the next frame.
   angle += 0.01f;

   if(angle >= 360)
      angle = 0.0f;
}


void SetView()
{
   // This will hold the view matrix.
   D3DXMATRIXA16 View;

   D3DXVECTOR3 Eye(0.0f, 0.0f, -5.0f);     // Position of view.
   D3DXVECTOR3 LookAt(0.0f, 0.0f, 1.0f);   // Point view is looking at.
   D3DXVECTOR3 Up(0.0f, 1.0f, 0.0f);       // Which way is up.

   // Set all the values this matrix will need to know (position, look at point, etc).
   D3DXMatrixLookAtLH(&View, &Eye, &LookAt, &Up);

   // Move the "view" to the position and values we set above.
   D3D_Device->SetTransform(D3DTS_VIEW, &View);
}


void RenderScene()
{
   // Clear the screen to black (You can change what color it changes to in D3DCOLOR_XRGB).
   D3D_Device->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER,
                     D3DCOLOR_XRGB(0,0,0), 1.0f, 0);
    
   // Tell Direct 3D we are now going to start drawing.
   D3D_Device->BeginScene();

      SetView();                       // Set the view matrix (view position).
      SetTranslation(0.0f, 0.0f, 1.0f);// Translate back.
      SetRotation();                   // Rotate scene.
      World = rot * trans;             // Create the world matrix.
   
      // Set the matrix.
      D3D_Device->SetTransform(D3DTS_WORLD, &World);

      // Set the material and draw the Teapot.
      D3D_Device->SetMaterial(&TeapotMaterial);
      Teapot->DrawSubset(0);
   
   // We are done drawing for this scene.
   D3D_Device->EndScene();
    
   // Swap the old frame with the new one.
   D3D_Device->Present(NULL, NULL, NULL, NULL);

   // This will get the frames per second for our app.
   GetFPS();
}


void GetFPS()
{
   // This will hold the number of frames that will go in the title bar.
   // Lets initialize this to 0;
   static char titleBar[25] = {0};

   // This will hold the number of frames per second in between seconds.
   static float FPS = 0.0f;

   // This will be used to check if a second has passed.
   float nextSecond = 0.0f;

   // This is the last second that occured.
   static float prevSecond = 0.0f;

   // Add to the frames per second every time this function is called.
   FPS++;

   // Get the second in millisecond then multiply it by 0.001 to convert this to
   // seconds.
   nextSecond = GetTickCount() * 0.001f;				

   // If the time we have now substracted from the previous second is greater than
   // or equal to 1 (i.e. if a second has passed) then we display the FPS number.
   if(nextSecond - prevSecond > 1.0f)
      {
         // Make the second we just got the previous second so we can used this the
         // next time the function is called.
	      prevSecond = nextSecond;
		   
         // print the FPS in the titlebar array.  Don't forget to cast the FPS so
         // what is printed is a whole number.  Or you can just print only 2 or
         // even no decimal places after the number.  What ever turns you on.
		   sprintf(titleBar, "%d FPS", int(FPS));

         // Set the window titlebar to the string in titleBar.
		   SetWindowText(g_hwnd, titleBar);

         // Reset the FPS counter.
         FPS = 0;
      }
}


void ShutdownDirect3D()
{
   // Here we release the Direct3D objects.
   if(D3D_Device != NULL)
      {
         D3D_Device->Release();
         D3D_Device = NULL;
      }

   if(Direct3D_Object != NULL)
      {
         Direct3D_Object->Release();
         Direct3D_Object = NULL;
      }
   
   if(Teapot != NULL)
      {
         Teapot->Release();
         Teapot = NULL;
      }
}


// Recap:

// Now we have the ability to keep track of the frames per second in an Direct3D
// application.


// Copyright September 2004
// All Rights Reserved!
// Allen Sherrod
// ProgrammingAce@UltimateGameProgramming.com
// www.UltimateGameProgramming.com